@@ -31,7 +31,7 @@ module Mixlib
       # Otherwise, doesn't print the time.
       def call(severity, time, progname, msg)
         if @@show_time
-          sprintf("[%s] %s: %s\n", time.iso8601(), severity, msg2str(msg))
+          sprintf("[%s] %s: %s\n", time.iso8601, severity, msg2str(msg))
         else
           sprintf("%s: %s\n", severity, msg2str(msg))
         end
@@ -42,15 +42,25 @@ module Mixlib
       # put through "object.inspect"
       def msg2str(msg)
         case msg
+        when ::Hash
+          if msg.has_key?(:err)
+            format_exception(msg[:err])
+          else
+            msg[:msg]
+          end
         when ::String
           msg
         when ::Exception
-          "#{ msg.message } (#{ msg.class })\n" <<
-            (msg.backtrace || []).join("\n")
+          format_exception(msg)
         else
           msg.inspect
         end
       end
+
+      def format_exception(msg)
+        "#{msg.message} (#{msg.class})\n" <<
+          (msg.backtrace || []).join("\n")
+      end
     end
   end
 end
