@@ -19,16 +19,26 @@
 
 require "tempfile"
 require "stringio"
-require File.expand_path(File.join(File.dirname(__FILE__), "..", "spec_helper"))
+require "spec_helper"
 
 class LoggerLike
   attr_accessor :level
-  attr_reader :messages
+  attr_reader :messages, :data
   def initialize
     @messages = ""
+    @data = []
   end
 
-  [:debug, :info, :warn, :error, :fatal].each do |method_name|
+  def add_data(severity, message = nil, progname = nil, data: {})
+    @messages << message
+    @data << data
+  end
+
+  def add(severity, message = nil, progname = nil, data: {})
+    @messages << message
+  end
+
+  %i{trace debug info warn error fatal}.each do |method_name|
     class_eval(<<-E)
       def #{method_name}(message)
         @messages << message
@@ -37,7 +47,7 @@ class LoggerLike
   end
 end
 
-describe Mixlib::Log do
+RSpec.describe Mixlib::Log do
 
   # Since we are testing class behaviour for an instance variable
   # that gets set once, we need to reset it prior to each example [cb]
@@ -64,6 +74,7 @@ describe Mixlib::Log do
   it "uses the logger provided when initialized with a logger like object" do
     logger = LoggerLike.new
     Logit.init(logger)
+    Logit.level = :debug
     Logit.debug "qux"
     expect(logger.messages).to match(/qux/)
   end
@@ -84,13 +95,14 @@ describe Mixlib::Log do
     expect(Logit.configured?).to be true
   end
 
-  it "should set the log level using the binding form,  with :debug, :info, :warn, :error, or :fatal" do
+  it "should set the log level using the binding form,  with :trace, :debug, :info, :warn, :error, or :fatal" do
     levels = {
-      :debug => Logger::DEBUG,
-      :info  => Logger::INFO,
-      :warn  => Logger::WARN,
-      :error => Logger::ERROR,
-      :fatal => Logger::FATAL,
+      trace: Mixlib::Log::TRACE,
+      debug: Mixlib::Log::DEBUG,
+      info: Mixlib::Log::INFO,
+      warn: Mixlib::Log::WARN,
+      error: Mixlib::Log::ERROR,
+      fatal: Mixlib::Log::FATAL,
     }
     levels.each do |symbol, constant|
       Logit.level = symbol
@@ -106,13 +118,14 @@ describe Mixlib::Log do
     expect(logdev.string).to match(/the_message/)
   end
 
-  it "should set the log level using the method form, with :debug, :info, :warn, :error, or :fatal" do
+  it "should set the log level using the method form, with :trace, :debug, :info, :warn, :error, or :fatal" do
     levels = {
-      :debug => Logger::DEBUG,
-      :info  => Logger::INFO,
-      :warn  => Logger::WARN,
-      :error => Logger::ERROR,
-      :fatal => Logger::FATAL,
+      trace: Mixlib::Log::TRACE,
+      debug: Mixlib::Log::DEBUG,
+      info: Mixlib::Log::INFO,
+      warn: Mixlib::Log::WARN,
+      error: Mixlib::Log::ERROR,
+      fatal: Mixlib::Log::FATAL,
     }
     levels.each do |symbol, constant|
       Logit.level(symbol)
@@ -121,17 +134,22 @@ describe Mixlib::Log do
   end
 
   it "should raise an ArgumentError if you try and set the level to something strange using the binding form" do
-    expect(lambda { Logit.level = :the_roots }).to raise_error(ArgumentError)
+    expect { Logit.level = :the_roots }.to raise_error(ArgumentError)
   end
 
   it "should raise an ArgumentError if you try and set the level to something strange using the method form" do
-    expect(lambda { Logit.level(:the_roots) }).to raise_error(ArgumentError)
+    expect { Logit.level(:the_roots) }.to raise_error(ArgumentError)
   end
 
   it "should pass other method calls directly to logger" do
-    Logit.level = :debug
-    expect(Logit).to be_debug
-    expect(lambda { Logit.debug("Gimme some sugar!") }).to_not raise_error
+    expect do
+      # this needs to be inside of the block because the level setting
+      # is causing the init, which grabs $stderr before rspec replaces
+      # it for output testing.
+      Logit.level = :debug
+      expect(Logit).to be_debug
+      Logit.debug("Gimme some sugar!")
+    end.to output(/DEBUG: Gimme some sugar!/).to_stdout
   end
 
   it "should pass add method calls directly to logger" do
@@ -139,12 +157,13 @@ describe Mixlib::Log do
     Logit.init(logdev)
     Logit.level = :debug
     expect(Logit).to be_debug
-    expect(lambda { Logit.add(Logger::DEBUG, "Gimme some sugar!") }).to_not raise_error
+    expect { Logit.add(Logger::DEBUG, "Gimme some sugar!") }.to_not raise_error
     expect(logdev.string).to match(/Gimme some sugar/)
   end
 
   it "should default to STDOUT if init is called with no arguments" do
     logger_mock = Struct.new(:formatter, :level).new
+    # intentionally STDOUT to avoid unfailable test
     expect(Logger).to receive(:new).with(STDOUT).and_return(logger_mock)
     Logit.init
   end
@@ -161,9 +180,10 @@ describe Mixlib::Log do
     ObjectSpace.each_object(File) do |f|
       opened_files_count_before += 1 unless f.closed?
     end
-    Logit.init("/tmp/logger.log")
-    Logit.init("/tmp/logger.log")
-    Logit.init("/tmp/logger.log")
+    name = File.join(Dir.tmpdir, "logger.log")
+    Logit.init(name)
+    Logit.init(name)
+    Logit.init(name)
     opened_files_count_after = 0
     ObjectSpace.each_object(File) do |f|
       opened_files_count_after += 1 unless f.closed?
@@ -176,15 +196,46 @@ describe Mixlib::Log do
     ObjectSpace.each_object(File) do |f|
       opened_files_count_before += 1 unless f.closed?
     end
-    file = File.open("/tmp/logger.log")
-    Logit.init(file)
-    Logit.init(file)
-    Logit.init(file)
-    opened_files_count_after = 0
-    ObjectSpace.each_object(File) do |f|
-      opened_files_count_after += 1 unless f.closed?
+    Tempfile.open("chef-test-log") do |file|
+      Logit.init(file)
+      Logit.init(file)
+      Logit.init(file)
+      opened_files_count_after = 0
+      ObjectSpace.each_object(File) do |f|
+        opened_files_count_after += 1 unless f.closed?
+      end
+      expect(opened_files_count_after).to eq(opened_files_count_before + 1)
     end
-    expect(opened_files_count_after).to eq(opened_files_count_before + 1)
   end
 
+  it "should return nil from its logging methods" do
+    # intentionally STDOUT to avoid unfailable test
+    expect(Logger).to receive(:new).with(STDOUT) { double("a-quiet-logger").as_null_object }
+    Logit.init
+
+    aggregate_failures "returns nil from logging method" do
+      expect(Logit.trace("hello")).to be_nil
+      expect(Logit.debug("hello")).to be_nil
+      expect(Logit.info("hello")).to be_nil
+      expect(Logit.warn("hello")).to be_nil
+      expect(Logit.error("hello")).to be_nil
+      expect(Logit.fatal("hello")).to be_nil
+    end
+  end
+
+  it "should set metadata correctly" do
+    Logit.metadata = { test: "data" }
+    expect(Logit.metadata).to eql({ test: "data" })
+  end
+
+  it "should format :trace level messages with TRACE: label" do
+    logdev = StringIO.new
+    Logit.init(logdev)
+    Logit.level = :trace
+    Logit.trace("this is a log message")
+    aggregate_failures do
+      expect(logdev.string).to_not match(/ANY:/)
+      expect(logdev.string).to match(/TRACE:/)
+    end
+  end
 end
